//
// Fur example for OpenGL
//
// Author: Alex V. Boreskoff <steps3d@gmail.com>, <steps3d@narod.ru>
//

#include	"libExt.h"

#ifdef	MACOSX
	#include	<GLUT/glut.h>
#else
	#include	<glut.h>
#endif

#include	<stdio.h>
#include	<stdlib.h>

#include	"libTexture.h"
#include	"Vector3D.h"
#include	"Vector2D.h"
#include	"Torus.h"
#include	"VertexProgram.h"
#include	"FragmentProgram.h"

Vector3D	eye   ( 6, 5, 6 );			// camera position
Vector3D	light ( 5, 0, 4 );			// light position
unsigned	decalMap;
float	 	angle = 0;
Torus		torus ( 1, 3, 30, 30 );
Vector3D	rot ( 0, 0, 0 );
int			mouseOldX = 0;
int			mouseOldY = 0;
unsigned	furTex [16];
unsigned	furTan [16];

VertexProgram	vertexProgram;
FragmentProgram	fragmentProgram;

void init ()
{
	glClearColor ( 0.0, 0.0, 0.0, 1.0 );
	glEnable     ( GL_DEPTH_TEST );
	glDepthFunc  ( GL_LEQUAL );

	glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
	glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void display ()
{
	glClear      ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	glPushMatrix ();

	glPushMatrix ();

	int	maxAniso;

	glGetIntegerv(GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &maxAniso );

	if ( maxAniso < 16 )
		maxAniso = 16;


												// draw the light
	glMatrixMode ( GL_MODELVIEW );
	glPushMatrix ();

	glTranslatef       ( light.x, light.y, light.z );
	glActiveTextureARB ( GL_TEXTURE0_ARB );
	glDisable          ( GL_TEXTURE_2D );
	glColor4f          ( 1, 1, 1, 1 );
	glutSolidSphere    ( 0.1f, 15, 15 );
	glPopMatrix        ();

												// set matrix0
	glMatrixMode   ( GL_MATRIX0_ARB );
	glLoadIdentity ();
    glRotatef      ( rot.x, 1, 0, 0 );
    glRotatef      ( rot.y, 0, 1, 0 );
    glRotatef      ( rot.z, 0, 0, 1 );

                               // draw torus itself
	glActiveTextureARB ( GL_TEXTURE0_ARB );
	glEnable           ( GL_TEXTURE_2D );
	glBindTexture      ( GL_TEXTURE_2D, decalMap );

	glMatrixMode   ( GL_MODELVIEW );
	glPushMatrix   ();

    glRotatef    ( rot.x, 1, 0, 0 );
    glRotatef    ( rot.y, 0, 1, 0 );
    glRotatef    ( rot.z, 0, 0, 1 );

	glEnable   ( GL_CULL_FACE );
	glCullFace ( GL_BACK );

	torus.draw  ();

								// now start drawing shells
	float	scale     = 0.5;
	float	shadowMin = 0.2;
	float	shadowMax = 0.5;

	glDisable      ( GL_CULL_FACE );
	glEnable       ( GL_BLEND );
	glBlendFunc    ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );

								// draw shells from innermost to outermost
	vertexProgram  .enable ();
	vertexProgram  .bind   ();
	fragmentProgram.enable ();
	fragmentProgram.bind   ();

	for ( int i = 0; i < 16; i++ )
	{
		float	t      = (float) i / 16.0;
		float	shadow = shadowMin * (1 - t) + shadowMax * t;

		glActiveTextureARB ( GL_TEXTURE0_ARB );
		glBindTexture      ( GL_TEXTURE_2D, furTex [i] );
		glTexParameteri    ( GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, maxAniso );

		glActiveTextureARB ( GL_TEXTURE1_ARB );
		glBindTexture      ( GL_TEXTURE_2D, furTan [i] );
		glTexParameteri    ( GL_TEXTURE_2D, GL_TEXTURE_MAX_ANISOTROPY_EXT, maxAniso );

		glColor4f       ( 1, 1, 1, shadow );

		vertexProgram.local [2] = Vector3D ( t * scale, 0, 0 );

		torus.draw ( 0 );
	}

	vertexProgram  .disable ();
	fragmentProgram.disable ();

	glDisable ( GL_BLEND );
	glDisable ( GL_REGISTER_COMBINERS_NV );

	glPopMatrix     ();
	glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
   gluLookAt      ( eye.x, eye.y, eye.z,	// eye
					0, 0, 0,				// center
					0, 0, 1 );				// up
}

void motion ( int x, int y )
{
    rot.y -= ((mouseOldY - y) * 180.0f) / 200.0f;
    rot.z -= ((mouseOldX - x) * 180.0f) / 200.0f;
    rot.x  = 0;

    if ( rot.z > 360 )
		rot.z -= 360;

	if ( rot.z < -360 )
		rot.z += 360;

    if ( rot.y > 360 )
		rot.y -= 360;

	if ( rot.y < -360 )
        rot.y += 360;

    mouseOldX = x;
    mouseOldY = y;

	glutPostRedisplay ();
}

void mouse ( int button, int state, int x, int y )
{
    if ( state == GLUT_DOWN )
    {
        mouseOldX = x;
        mouseOldY = y;
	}
}

void key ( unsigned char key, int x, int y )
{
	if ( key == 27 || key == 'q' || key == 'Q' )		// quit requested
    	exit ( 0 );
}

void	animate ()
{
	angle  = 0.004f * glutGet ( GLUT_ELAPSED_TIME );

	light.x = 3*cos ( angle );
	light.y = 2*sin ( angle );
	light.z = 4 + 0.4*sin ( angle / 3 );

										// setup data
	vertexProgram.bind ();

	vertexProgram.local [0] = eye;
	vertexProgram.local [1] = light;

	vertexProgram.unbind ();

	glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
								// initialize glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize  ( 500, 500 );


								// create window
	glutCreateWindow ( "OpenGL fur demo 5 (per-pixel lighting)" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
	glutKeyboardFunc ( key     );
	glutMouseFunc    ( mouse   );
	glutMotionFunc   ( motion  );
	glutIdleFunc     ( animate );

	init           ();
	initExtensions ();

	if ( !isExtensionSupported ( "GL_ARB_vertex_program" ) )
	{
		printf ( "GL_ARB_vertex_program NOT supported" );

		return 1;
	}

	if ( !isExtensionSupported ( "GL_ARB_fragment_program" ) )
	{
		printf ( "GL_ARB_fragment_program NOT supported" );

		return 2;
	}

	decalMap = createTexture2D ( true,  "../../Textures/block.bmp" );

								// load fur textures
	for ( int i = 0; i < 16; i++ )
	{
		char	name1 [1218];
		char	name2 [1218];

		sprintf ( name1, "Textures/fur-map2-%02d.png", i );
		sprintf ( name2, "Textures/fur-tangent-map-%02d.png", i );

		furTex [i] = createTexture2D ( true, name1 );
		furTan [i] = createTexture2D ( true, name2 );
	}

	if ( !vertexProgram.load ( "fur-3.vp" ) )
	{
		printf ( "Error loading vertex program:\n %s\n", vertexProgram.getErrorString ().c_str () );

		return 1;
	}

	if ( !fragmentProgram.load ( "fur-3.fp" ) )
	{
		printf ( "Error loading fragment program:\n %s\n", fragmentProgram.getErrorString ().c_str () );

		return 2;
	}

	glutMainLoop ();

	return 0;
}
