//
// Fur example for OpenGL
//
// Author: Alex V. Boreskoff <alexboreskoff@mtu-net.ru>, <steps3d@narod.ru>
//

#include	"libExt.h"

#include	<glut.h>
#include	<stdio.h>
#include	<stdlib.h>

#include	"libTexture.h"
#include	"Vector3D.h"
#include	"Vector2D.h"
#include	"MeshUtils.h"
#include	"GlslProgram.h"

Vector3D	eye   ( 6, 5, 6 );			// camera position
Vector3D	light ( 5, 0, 4 );			// light position
unsigned	decalMap;
float	 	angle = 0;
Vector3D	rot ( 0, 0, 0 );
int			mouseOldX = 0;
int			mouseOldY = 0;
unsigned	furMap;
unsigned	anisoMap;
int			numLayers = 16;
int			maxAniso;
float		alphaScale = 0.7;
	
GlslProgram	program;
Mesh      * torus;

void init ()
{
	glClearColor ( 0.0, 0.0, 0.0, 1.0 );
	glEnable     ( GL_DEPTH_TEST );
	glDepthFunc  ( GL_LEQUAL );

	glHint ( GL_POLYGON_SMOOTH_HINT,         GL_NICEST );
	glHint ( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );
}

void display ()
{
	glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
												// draw the light
	glMatrixMode ( GL_MODELVIEW );
	glPushMatrix ();

	glTranslatef       ( light.x, light.y, light.z );
	glColor4f          ( 1, 1, 1, 1 );
	glDisable          ( GL_TEXTURE_2D );
	glutSolidSphere    ( 0.1f, 15, 15 );
	glPopMatrix        ();

	glMatrixMode   ( GL_MODELVIEW );
	glPushMatrix   ();

    glRotatef    ( rot.x, 1, 0, 0 );
    glRotatef    ( rot.y, 0, 1, 0 );
    glRotatef    ( rot.z, 0, 0, 1 );

                               // draw torus itself
	glActiveTextureARB ( GL_TEXTURE0_ARB );
	glEnable           ( GL_TEXTURE_2D );
	glBindTexture      ( GL_TEXTURE_2D, decalMap );

	torus -> render  ();
	
	glDisable ( GL_TEXTURE_2D );
	
								// now start drawing shells
	float	shadowMin = 0.2;
	float	shadowMax = 0.5;

	glActiveTextureARB ( GL_TEXTURE1_ARB );
	glEnable           ( GL_TEXTURE_2D );
	glBindTexture      ( GL_TEXTURE_2D, anisoMap );

	glActiveTextureARB ( GL_TEXTURE0_ARB );
	glEnable           ( GL_TEXTURE_2D );
	glBindTexture      ( GL_TEXTURE_2D, furMap );

	
								// draw shells from innermost to outermost
	glEnable           ( GL_BLEND );
	glBlendFunc        ( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	
	program.bind ();
	
	for ( int i = 0; i < numLayers; i++ )
	{
		float	t      = (float) i / 16.0;
		float	shadow = 0.7*(shadowMin * (1 - t) + shadowMax * t);

		glColor4f ( shadow, shadow, shadow, shadow );

		program.setUniformFloat ( "tl", t * alphaScale );

		torus -> render ();
	}

	program.unbind ();
	
	glDisable ( GL_BLEND );

	glPopMatrix     ();
	glutSwapBuffers ();
}

void reshape ( int w, int h )
{
   glViewport     ( 0, 0, (GLsizei)w, (GLsizei)h );
   glMatrixMode   ( GL_PROJECTION );
   											// factor all camera ops into projection matrix
   glLoadIdentity ();
   gluPerspective ( 60.0, (GLfloat)w/(GLfloat)h, 1.0, 60.0 );
   gluLookAt      ( eye.x, eye.y, eye.z,	// eye
					0, 0, 0,				// center
					0, 0, 1 );				// up

   glMatrixMode   ( GL_MODELVIEW );
   glLoadIdentity ();
}

void motion ( int x, int y )
{
    rot.y -= ((mouseOldY - y) * 180.0f) / 200.0f;
    rot.z -= ((mouseOldX - x) * 180.0f) / 200.0f;
    rot.x  = 0;

    if ( rot.z > 360 )
		rot.z -= 360;

	if ( rot.z < -360 )
		rot.z += 360;

    if ( rot.y > 360 )
		rot.y -= 360;

	if ( rot.y < -360 )
        rot.y += 360;

    mouseOldX = x;
    mouseOldY = y;

	glutPostRedisplay ();
}

void mouse ( int button, int state, int x, int y )
{
    if ( state == GLUT_DOWN )
    {
        mouseOldX = x;
        mouseOldY = y;
	}
}

void key ( unsigned char key, int x, int y )
{
	if ( key == 27 || key == 'q' || key == 'Q' )		// quit requested
    	exit ( 0 );
		
	if ( key == '+' )
		numLayers++;
	else
	if ( key == '-' )
		numLayers--;
	else
	if ( key == '*' )
		alphaScale += 0.03;
	else
	if ( key == '/' )
		alphaScale -= 0.03;
}

void	animate ()
{
	angle  = 0.004f * glutGet ( GLUT_ELAPSED_TIME );

	light.x = 3*cos ( angle );
	light.y = 2*sin ( angle );
	light.z = 3 + 0.3 * sin ( angle / 3 );

	program.bind             ();
	program.setUniformVector ( "eyePos",   Vector4D ( eye,   1 ) );
	program.setUniformVector ( "lightPos", Vector4D ( light, 1 ) );
	program.setUniformFloat  ( "time",     angle                 );
	program.unbind           ();

	glutPostRedisplay ();
}

int main ( int argc, char * argv [] )
{
								// initialize glut
	glutInit            ( &argc, argv );
	glutInitDisplayMode ( GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH );
	glutInitWindowSize  ( 500, 500 );

								// create window
	glutCreateWindow ( "OpenGL GLSL fur demo" );

								// register handlers
	glutDisplayFunc  ( display );
	glutReshapeFunc  ( reshape );
	glutKeyboardFunc ( key     );
	glutMouseFunc    ( mouse   );
	glutMotionFunc   ( motion  );
	glutIdleFunc     ( animate );	

	init           ();
	initExtensions ();

	decalMap = createTexture2D ( true,  "../../Textures/block.bmp" );
	anisoMap = createTexture2D ( true, "aniso-table.bmp" );
	furMap   = createTexture2D ( true, "Textures/fur-map-00.png" );

	if ( !GlslProgram :: isSupported () )
	{
		printf ( "GLSL not supported.\n" );

		return 1;
	}

	if ( !program.loadShaders ( "fur-6.vsh", "fur-6.fsh" ) )
	{
		printf ( "Error loading shaders:\n%s\n", program.getLog ().c_str () );

		return 2;
	}
	
	program.bind       ();
    program.setTexture ( "furMap",   0 );
    program.setTexture ( "anisoMap", 1 );
	program.unbind     ();

	torus = makeTorus ( 1, 3, 30, 30 );

	torus -> addCoordAssignment ( Mesh :: tagVertex,   Mesh :: tagVertex );
	torus -> addCoordAssignment ( Mesh :: tagNormal,   Mesh :: tagNormal );
	torus -> addCoordAssignment ( Mesh :: tagTexCoord, Mesh :: tagTex0   );
	torus -> addCoordAssignment ( Mesh :: tagTangent,  Mesh :: tagTex1   );
	torus -> addCoordAssignment ( Mesh :: tagBinormal, Mesh :: tagTex2   );
	torus -> createBuffers      ();

	glGetIntegerv ( GL_MAX_TEXTURE_MAX_ANISOTROPY_EXT, &maxAniso );

	if ( maxAniso < 16 )
		maxAniso = 16;
	
	printf ( "Use + and - keys to change number of fur layers\nUse * and / to control alphaScale\n" );
	
	glutMainLoop ();

	return 0;
}
