//
// Simple anisotropic lighting vertex shader
// Using lookup table for diffuse and specular coefficients
//

uniform  float	  tl;
uniform  float	  time;
uniform	 vec4	  lightPos;
uniform	 vec4	  eyePos;

varying vec3 lt;
varying vec3 ht;
varying vec4 color;
varying vec3 furDir;

void main ()
{
	vec4		 pp = gl_Vertex + tl * vec4 ( gl_Normal, 0.0 );
	float		 d1 = 0.01 * sin ( 0.731*time ) * tl;
	float		 d2 = 0.03 * sin ( 0.231*time ) * tl;
	
	vec3	p = (gl_ModelViewMatrix * gl_Vertex).xyz;						// transformed point to world space
	vec3	l = normalize ( lightPos.xyz - p );				// vector to light source
	vec3	v = normalize ( eyePos.xyz   - p );				// vector to the eye
	vec3	h = l + v;
	vec3	n = gl_NormalMatrix * gl_Normal;				// transformed n
	vec3 	t = gl_NormalMatrix * gl_MultiTexCoord1.xyz;	// transformed t
	vec3 	b = gl_NormalMatrix * gl_MultiTexCoord2.xyz;	// transformed b

	color    = gl_Color;													
	lt       = vec3 ( dot ( l, t ), dot ( l, b ), dot ( l, n ) );
	ht       = vec3 ( dot ( h, t ), dot ( h, b ), dot ( h, n ) );
	furDir   = vec3 ( d1, d2, 1.0 );
	
	gl_Position     = gl_ModelViewProjectionMatrix * pp;
	gl_TexCoord [0] = gl_MultiTexCoord0 + vec4 ( d1, d2, 0.0, 0.0 );
	
}
